# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["TestingCreateEvaluationParams"]


class TestingCreateEvaluationParams(TypedDict, total=False):
    eval_type: Required[Literal["EVALUATION", "FUZZING", "RED_TEAMING"]]

    name: Required[str]

    user_id: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel

__all__ = [
    "TestingUpdateResponse",
    "Content",
    "ContentBehavior",
    "ContentInputDetection",
    "ContentInputDetectionDetectorData",
    "ContentInputDetectionDetectorDataTextMatchingDetector",
    "ContentInputDetectionDetectorDataCategoryDetector",
    "ContentInputDetectionDetectorDataNaturalLanguageDetector",
    "ContentInputDetectionDetectorDataComparatorDetector",
    "ContentInputDetectionDetectorDataCustomDetector",
    "ContentInputMessage",
    "ContentInputMessageChatCompletionSystemMessageParam",
    "ContentInputMessageChatCompletionSystemMessageParamContentUnionMember1",
    "ContentInputMessageChatCompletionUserMessageParamOutput",
    "ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1",
    "ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartTextParam",
    "ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartImageParam",
    "ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartImageParamImageURL",
    "ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartInputAudioParam",
    "ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio",
    "ContentInputMessageChatCompletionAssistantMessageParamOutput",
    "ContentInputMessageChatCompletionAssistantMessageParamOutputAudio",
    "ContentInputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1",
    "ContentInputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1ChatCompletionContentPartTextParam",
    "ContentInputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1ChatCompletionContentPartRefusalParam",
    "ContentInputMessageChatCompletionAssistantMessageParamOutputFunctionCall",
    "ContentInputMessageChatCompletionAssistantMessageParamOutputToolCall",
    "ContentInputMessageChatCompletionAssistantMessageParamOutputToolCallFunction",
    "ContentInputMessageChatCompletionToolMessageParam",
    "ContentInputMessageChatCompletionToolMessageParamContentUnionMember1",
    "ContentInputMessageChatCompletionFunctionMessageParam",
    "ContentOutputDetection",
    "ContentOutputDetectionDetectorData",
    "ContentOutputDetectionDetectorDataTextMatchingDetector",
    "ContentOutputDetectionDetectorDataCategoryDetector",
    "ContentOutputDetectionDetectorDataNaturalLanguageDetector",
    "ContentOutputDetectionDetectorDataComparatorDetector",
    "ContentOutputDetectionDetectorDataCustomDetector",
    "ContentOutputMessage",
    "ContentOutputMessageChatCompletionSystemMessageParam",
    "ContentOutputMessageChatCompletionSystemMessageParamContentUnionMember1",
    "ContentOutputMessageChatCompletionUserMessageParamOutput",
    "ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1",
    "ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartTextParam",
    "ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartImageParam",
    "ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartImageParamImageURL",
    "ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartInputAudioParam",
    "ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio",
    "ContentOutputMessageChatCompletionAssistantMessageParamOutput",
    "ContentOutputMessageChatCompletionAssistantMessageParamOutputAudio",
    "ContentOutputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1",
    "ContentOutputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1ChatCompletionContentPartTextParam",
    "ContentOutputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1ChatCompletionContentPartRefusalParam",
    "ContentOutputMessageChatCompletionAssistantMessageParamOutputFunctionCall",
    "ContentOutputMessageChatCompletionAssistantMessageParamOutputToolCall",
    "ContentOutputMessageChatCompletionAssistantMessageParamOutputToolCallFunction",
    "ContentOutputMessageChatCompletionToolMessageParam",
    "ContentOutputMessageChatCompletionToolMessageParamContentUnionMember1",
    "ContentOutputMessageChatCompletionFunctionMessageParam",
]


class ContentBehavior(BaseModel):
    description: str


class ContentInputDetectionDetectorDataTextMatchingDetector(BaseModel):
    name: str

    regex: str

    user_id: str

    id: Optional[str] = None

    created: Optional[datetime] = None

    detector_type: Optional[Literal["TEXT_MATCHING"]] = None

    last_updated: Optional[datetime] = None


class ContentInputDetectionDetectorDataCategoryDetector(BaseModel):
    category: str

    name: str

    user_id: str

    id: Optional[str] = None

    created: Optional[datetime] = None

    detector_type: Optional[Literal["CATEGORY"]] = None

    last_updated: Optional[datetime] = None


class ContentInputDetectionDetectorDataNaturalLanguageDetector(BaseModel):
    name: str

    natural_language_content: str

    user_id: str

    id: Optional[str] = None

    created: Optional[datetime] = None

    detector_type: Optional[Literal["NATURAL_LANGUAGE"]] = None

    last_updated: Optional[datetime] = None


class ContentInputDetectionDetectorDataComparatorDetector(BaseModel):
    name: str

    type: Literal["EXACT_MATCH", "LANGUAGE_MODEL_SIMILARITY"]

    user_id: str

    id: Optional[str] = None

    created: Optional[datetime] = None

    detector_type: Optional[Literal["COMPARATOR"]] = None

    last_updated: Optional[datetime] = None


class ContentInputDetectionDetectorDataCustomDetector(BaseModel):
    name: str

    type: Literal["TIERED_DETECTOR"]

    user_id: str

    id: Optional[str] = None

    created: Optional[datetime] = None

    detector_type: Optional[Literal["CUSTOM"]] = None

    last_updated: Optional[datetime] = None


ContentInputDetectionDetectorData: TypeAlias = Annotated[
    Union[
        ContentInputDetectionDetectorDataTextMatchingDetector,
        ContentInputDetectionDetectorDataCategoryDetector,
        ContentInputDetectionDetectorDataNaturalLanguageDetector,
        ContentInputDetectionDetectorDataComparatorDetector,
        ContentInputDetectionDetectorDataCustomDetector,
        None,
    ],
    PropertyInfo(discriminator="detector_type"),
]


class ContentInputDetection(BaseModel):
    content_id: str

    detected: bool

    detector_id: str

    end_time: datetime

    score: float

    start_time: datetime

    detector_data: Optional[ContentInputDetectionDetectorData] = None


class ContentInputMessageChatCompletionSystemMessageParamContentUnionMember1(BaseModel):
    text: str

    type: Literal["text"]


class ContentInputMessageChatCompletionSystemMessageParam(BaseModel):
    content: Union[str, List[ContentInputMessageChatCompletionSystemMessageParamContentUnionMember1]]

    role: Literal["system"]

    name: Optional[str] = None


class ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartTextParam(
    BaseModel
):
    text: str

    type: Literal["text"]


class ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartImageParamImageURL(
    BaseModel
):
    url: str

    detail: Optional[Literal["auto", "low", "high"]] = None


class ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartImageParam(
    BaseModel
):
    image_url: ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartImageParamImageURL

    type: Literal["image_url"]


class ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio(
    BaseModel
):
    data: str

    format: Literal["wav", "mp3"]


class ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartInputAudioParam(
    BaseModel
):
    input_audio: ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio

    type: Literal["input_audio"]


ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1: TypeAlias = Union[
    ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartTextParam,
    ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartImageParam,
    ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartInputAudioParam,
]


class ContentInputMessageChatCompletionUserMessageParamOutput(BaseModel):
    content: Union[str, List[ContentInputMessageChatCompletionUserMessageParamOutputContentUnionMember1]]

    role: Literal["user"]

    name: Optional[str] = None


class ContentInputMessageChatCompletionAssistantMessageParamOutputAudio(BaseModel):
    id: str


class ContentInputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1ChatCompletionContentPartTextParam(
    BaseModel
):
    text: str

    type: Literal["text"]


class ContentInputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1ChatCompletionContentPartRefusalParam(
    BaseModel
):
    refusal: str

    type: Literal["refusal"]


ContentInputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1: TypeAlias = Union[
    ContentInputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1ChatCompletionContentPartTextParam,
    ContentInputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1ChatCompletionContentPartRefusalParam,
]


class ContentInputMessageChatCompletionAssistantMessageParamOutputFunctionCall(BaseModel):
    arguments: str

    name: str


class ContentInputMessageChatCompletionAssistantMessageParamOutputToolCallFunction(BaseModel):
    arguments: str

    name: str


class ContentInputMessageChatCompletionAssistantMessageParamOutputToolCall(BaseModel):
    id: str

    function: ContentInputMessageChatCompletionAssistantMessageParamOutputToolCallFunction

    type: Literal["function"]


class ContentInputMessageChatCompletionAssistantMessageParamOutput(BaseModel):
    role: Literal["assistant"]

    audio: Optional[ContentInputMessageChatCompletionAssistantMessageParamOutputAudio] = None

    content: Union[str, List[ContentInputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1], None] = (
        None
    )

    function_call: Optional[ContentInputMessageChatCompletionAssistantMessageParamOutputFunctionCall] = None

    name: Optional[str] = None

    refusal: Optional[str] = None

    tool_calls: Optional[List[ContentInputMessageChatCompletionAssistantMessageParamOutputToolCall]] = None


class ContentInputMessageChatCompletionToolMessageParamContentUnionMember1(BaseModel):
    text: str

    type: Literal["text"]


class ContentInputMessageChatCompletionToolMessageParam(BaseModel):
    content: Union[str, List[ContentInputMessageChatCompletionToolMessageParamContentUnionMember1]]

    role: Literal["tool"]

    tool_call_id: str


class ContentInputMessageChatCompletionFunctionMessageParam(BaseModel):
    content: Optional[str] = None

    name: str

    role: Literal["function"]


ContentInputMessage: TypeAlias = Union[
    ContentInputMessageChatCompletionSystemMessageParam,
    ContentInputMessageChatCompletionUserMessageParamOutput,
    ContentInputMessageChatCompletionAssistantMessageParamOutput,
    ContentInputMessageChatCompletionToolMessageParam,
    ContentInputMessageChatCompletionFunctionMessageParam,
]


class ContentOutputDetectionDetectorDataTextMatchingDetector(BaseModel):
    name: str

    regex: str

    user_id: str

    id: Optional[str] = None

    created: Optional[datetime] = None

    detector_type: Optional[Literal["TEXT_MATCHING"]] = None

    last_updated: Optional[datetime] = None


class ContentOutputDetectionDetectorDataCategoryDetector(BaseModel):
    category: str

    name: str

    user_id: str

    id: Optional[str] = None

    created: Optional[datetime] = None

    detector_type: Optional[Literal["CATEGORY"]] = None

    last_updated: Optional[datetime] = None


class ContentOutputDetectionDetectorDataNaturalLanguageDetector(BaseModel):
    name: str

    natural_language_content: str

    user_id: str

    id: Optional[str] = None

    created: Optional[datetime] = None

    detector_type: Optional[Literal["NATURAL_LANGUAGE"]] = None

    last_updated: Optional[datetime] = None


class ContentOutputDetectionDetectorDataComparatorDetector(BaseModel):
    name: str

    type: Literal["EXACT_MATCH", "LANGUAGE_MODEL_SIMILARITY"]

    user_id: str

    id: Optional[str] = None

    created: Optional[datetime] = None

    detector_type: Optional[Literal["COMPARATOR"]] = None

    last_updated: Optional[datetime] = None


class ContentOutputDetectionDetectorDataCustomDetector(BaseModel):
    name: str

    type: Literal["TIERED_DETECTOR"]

    user_id: str

    id: Optional[str] = None

    created: Optional[datetime] = None

    detector_type: Optional[Literal["CUSTOM"]] = None

    last_updated: Optional[datetime] = None


ContentOutputDetectionDetectorData: TypeAlias = Annotated[
    Union[
        ContentOutputDetectionDetectorDataTextMatchingDetector,
        ContentOutputDetectionDetectorDataCategoryDetector,
        ContentOutputDetectionDetectorDataNaturalLanguageDetector,
        ContentOutputDetectionDetectorDataComparatorDetector,
        ContentOutputDetectionDetectorDataCustomDetector,
        None,
    ],
    PropertyInfo(discriminator="detector_type"),
]


class ContentOutputDetection(BaseModel):
    content_id: str

    detected: bool

    detector_id: str

    end_time: datetime

    score: float

    start_time: datetime

    detector_data: Optional[ContentOutputDetectionDetectorData] = None


class ContentOutputMessageChatCompletionSystemMessageParamContentUnionMember1(BaseModel):
    text: str

    type: Literal["text"]


class ContentOutputMessageChatCompletionSystemMessageParam(BaseModel):
    content: Union[str, List[ContentOutputMessageChatCompletionSystemMessageParamContentUnionMember1]]

    role: Literal["system"]

    name: Optional[str] = None


class ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartTextParam(
    BaseModel
):
    text: str

    type: Literal["text"]


class ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartImageParamImageURL(
    BaseModel
):
    url: str

    detail: Optional[Literal["auto", "low", "high"]] = None


class ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartImageParam(
    BaseModel
):
    image_url: ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartImageParamImageURL

    type: Literal["image_url"]


class ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio(
    BaseModel
):
    data: str

    format: Literal["wav", "mp3"]


class ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartInputAudioParam(
    BaseModel
):
    input_audio: ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio

    type: Literal["input_audio"]


ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1: TypeAlias = Union[
    ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartTextParam,
    ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartImageParam,
    ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1ChatCompletionContentPartInputAudioParam,
]


class ContentOutputMessageChatCompletionUserMessageParamOutput(BaseModel):
    content: Union[str, List[ContentOutputMessageChatCompletionUserMessageParamOutputContentUnionMember1]]

    role: Literal["user"]

    name: Optional[str] = None


class ContentOutputMessageChatCompletionAssistantMessageParamOutputAudio(BaseModel):
    id: str


class ContentOutputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1ChatCompletionContentPartTextParam(
    BaseModel
):
    text: str

    type: Literal["text"]


class ContentOutputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1ChatCompletionContentPartRefusalParam(
    BaseModel
):
    refusal: str

    type: Literal["refusal"]


ContentOutputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1: TypeAlias = Union[
    ContentOutputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1ChatCompletionContentPartTextParam,
    ContentOutputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1ChatCompletionContentPartRefusalParam,
]


class ContentOutputMessageChatCompletionAssistantMessageParamOutputFunctionCall(BaseModel):
    arguments: str

    name: str


class ContentOutputMessageChatCompletionAssistantMessageParamOutputToolCallFunction(BaseModel):
    arguments: str

    name: str


class ContentOutputMessageChatCompletionAssistantMessageParamOutputToolCall(BaseModel):
    id: str

    function: ContentOutputMessageChatCompletionAssistantMessageParamOutputToolCallFunction

    type: Literal["function"]


class ContentOutputMessageChatCompletionAssistantMessageParamOutput(BaseModel):
    role: Literal["assistant"]

    audio: Optional[ContentOutputMessageChatCompletionAssistantMessageParamOutputAudio] = None

    content: Union[
        str, List[ContentOutputMessageChatCompletionAssistantMessageParamOutputContentUnionMember1], None
    ] = None

    function_call: Optional[ContentOutputMessageChatCompletionAssistantMessageParamOutputFunctionCall] = None

    name: Optional[str] = None

    refusal: Optional[str] = None

    tool_calls: Optional[List[ContentOutputMessageChatCompletionAssistantMessageParamOutputToolCall]] = None


class ContentOutputMessageChatCompletionToolMessageParamContentUnionMember1(BaseModel):
    text: str

    type: Literal["text"]


class ContentOutputMessageChatCompletionToolMessageParam(BaseModel):
    content: Union[str, List[ContentOutputMessageChatCompletionToolMessageParamContentUnionMember1]]

    role: Literal["tool"]

    tool_call_id: str


class ContentOutputMessageChatCompletionFunctionMessageParam(BaseModel):
    content: Optional[str] = None

    name: str

    role: Literal["function"]


ContentOutputMessage: TypeAlias = Union[
    ContentOutputMessageChatCompletionSystemMessageParam,
    ContentOutputMessageChatCompletionUserMessageParamOutput,
    ContentOutputMessageChatCompletionAssistantMessageParamOutput,
    ContentOutputMessageChatCompletionToolMessageParam,
    ContentOutputMessageChatCompletionFunctionMessageParam,
]


class Content(BaseModel):
    id: Optional[str] = None

    algorithm: Optional[str] = None

    behavior: Optional[ContentBehavior] = None

    content_group_ids: Optional[List[str]] = None

    content_type: Optional[Literal["BASE", "TEST", "EXPERIMENT"]] = None

    end: Optional[datetime] = None

    generate_end_time: Optional[datetime] = None

    generate_start_time: Optional[datetime] = None

    input_detections: Optional[List[ContentInputDetection]] = None

    input_messages: Optional[List[ContentInputMessage]] = None

    metadata: Optional[object] = None

    output_detections: Optional[List[ContentOutputDetection]] = None

    output_messages: Optional[List[ContentOutputMessage]] = None

    start: Optional[datetime] = None

    status: Optional[
        Literal["GENERATING_CONTENT", "AWAITING_RESPONSE", "ANALYZING_RESPONSE", "COMPLETE", "DISCARDED"]
    ] = None
    """Status for a single content object in a test."""

    test_id: Optional[str] = None

    time: Optional[datetime] = None

    user_id: Optional[str] = None


class TestingUpdateResponse(BaseModel):
    __test__ = False
    status: Literal["COMPLETE", "ERROR", "RUNNING", "STEP_COMPLETE", "STOPPED"]
    """Status for the overall test."""

    id: Optional[str] = None

    contents: Optional[List[Content]] = None

    test_id: Optional[str] = None

    time: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["TestingStartResponse"]


class TestingStartResponse(BaseModel):
    __test__ = False
    status: Literal["COMPLETE", "ERROR", "RUNNING", "STEP_COMPLETE", "STOPPED"]
    """Status for the overall test."""

    test_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["TestingUpdateEvaluationParams"]


class TestingUpdateEvaluationParams(TypedDict, total=False):
    eval_id: Required[str]

    status: Required[Literal["COMPLETE", "ERROR", "RUNNING", "STOPPED", "PENDING"]]

    user_id: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = ["MonitoringLogParams", "Span", "SpanScore", "Trace"]


class Span(TypedDict, total=False):
    trace_id: Required[str]

    id: str

    caller_id: Optional[str]

    end: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    eval_id: str

    inputs: object

    metadata: object

    name: str

    outputs: object

    parent_id: Optional[str]

    scores: Iterable[SpanScore]

    span_type: Optional[Literal["DETECTOR", "JUDGE", "APP", "MODEL", "FUNCTION", "SCORER"]]

    start: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    tags: object

    user_id: str


class SpanScore(TypedDict, total=False):
    judge_id: Required[str]

    score: Required[float]

    judge_name: Optional[str]

    label: Optional[str]


class Trace(TypedDict, total=False):
    root_span: Required[str]

    id: str

    end: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    eval_id: str

    name: str

    start: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    user_id: str


MonitoringLogParams: TypeAlias = Union[Span, Trace]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional

import httpx

from ..types import judge_call_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.judge_call_response import JudgeCallResponse

__all__ = ["JudgesResource", "AsyncJudgesResource"]


class JudgesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> JudgesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/haizelabs/haizelabs-python#accessing-raw-response-data-eg-headers
        """
        return JudgesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> JudgesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/haizelabs/haizelabs-python#with_streaming_response
        """
        return JudgesResourceWithStreamingResponse(self)

    def call(
        self,
        *,
        judge_ids: List[str],
        id: str | NotGiven = NOT_GIVEN,
        behavior: Optional[str] | NotGiven = NOT_GIVEN,
        content: Optional[judge_call_params.Content] | NotGiven = NOT_GIVEN,
        content_id: Optional[str] | NotGiven = NOT_GIVEN,
        judge_input: bool | NotGiven = NOT_GIVEN,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> JudgeCallResponse:
        """
        Calls a judge.

        Args:
          content: A single piece of content in a test.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/judges/call",
            body=maybe_transform(
                {
                    "judge_ids": judge_ids,
                    "id": id,
                    "behavior": behavior,
                    "content": content,
                    "content_id": content_id,
                    "judge_input": judge_input,
                    "user_id": user_id,
                },
                judge_call_params.JudgeCallParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JudgeCallResponse,
        )


class AsyncJudgesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncJudgesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/haizelabs/haizelabs-python#accessing-raw-response-data-eg-headers
        """
        return AsyncJudgesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncJudgesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/haizelabs/haizelabs-python#with_streaming_response
        """
        return AsyncJudgesResourceWithStreamingResponse(self)

    async def call(
        self,
        *,
        judge_ids: List[str],
        id: str | NotGiven = NOT_GIVEN,
        behavior: Optional[str] | NotGiven = NOT_GIVEN,
        content: Optional[judge_call_params.Content] | NotGiven = NOT_GIVEN,
        content_id: Optional[str] | NotGiven = NOT_GIVEN,
        judge_input: bool | NotGiven = NOT_GIVEN,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> JudgeCallResponse:
        """
        Calls a judge.

        Args:
          content: A single piece of content in a test.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/judges/call",
            body=await async_maybe_transform(
                {
                    "judge_ids": judge_ids,
                    "id": id,
                    "behavior": behavior,
                    "content": content,
                    "content_id": content_id,
                    "judge_input": judge_input,
                    "user_id": user_id,
                },
                judge_call_params.JudgeCallParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JudgeCallResponse,
        )


class JudgesResourceWithRawResponse:
    def __init__(self, judges: JudgesResource) -> None:
        self._judges = judges

        self.call = to_raw_response_wrapper(
            judges.call,
        )


class AsyncJudgesResourceWithRawResponse:
    def __init__(self, judges: AsyncJudgesResource) -> None:
        self._judges = judges

        self.call = async_to_raw_response_wrapper(
            judges.call,
        )


class JudgesResourceWithStreamingResponse:
    def __init__(self, judges: JudgesResource) -> None:
        self._judges = judges

        self.call = to_streamed_response_wrapper(
            judges.call,
        )


class AsyncJudgesResourceWithStreamingResponse:
    def __init__(self, judges: AsyncJudgesResource) -> None:
        self._judges = judges

        self.call = async_to_streamed_response_wrapper(
            judges.call,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal, overload

import httpx

from ..types import monitoring_log_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    required_args,
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options

__all__ = ["MonitoringResource", "AsyncMonitoringResource"]


class MonitoringResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MonitoringResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/haizelabs/haizelabs-python#accessing-raw-response-data-eg-headers
        """
        return MonitoringResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MonitoringResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/haizelabs/haizelabs-python#with_streaming_response
        """
        return MonitoringResourceWithStreamingResponse(self)

    @overload
    def log(
        self,
        *,
        trace_id: str,
        id: str | NotGiven = NOT_GIVEN,
        caller_id: Optional[str] | NotGiven = NOT_GIVEN,
        end: Union[str, datetime] | NotGiven = NOT_GIVEN,
        eval_id: str | NotGiven = NOT_GIVEN,
        inputs: object | NotGiven = NOT_GIVEN,
        metadata: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        outputs: object | NotGiven = NOT_GIVEN,
        parent_id: Optional[str] | NotGiven = NOT_GIVEN,
        scores: Iterable[monitoring_log_params.SpanScore] | NotGiven = NOT_GIVEN,
        span_type: Optional[Literal["DETECTOR", "JUDGE", "APP", "MODEL", "FUNCTION", "SCORER"]] | NotGiven = NOT_GIVEN,
        start: Union[str, datetime] | NotGiven = NOT_GIVEN,
        tags: object | NotGiven = NOT_GIVEN,
        user_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        logs a span

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def log(
        self,
        *,
        root_span: str,
        id: str | NotGiven = NOT_GIVEN,
        end: Union[str, datetime] | NotGiven = NOT_GIVEN,
        eval_id: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        start: Union[str, datetime] | NotGiven = NOT_GIVEN,
        user_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        logs a span

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["trace_id"], ["root_span"])
    def log(
        self,
        *,
        trace_id: str | NotGiven = NOT_GIVEN,
        id: str | NotGiven = NOT_GIVEN,
        caller_id: Optional[str] | NotGiven = NOT_GIVEN,
        end: Union[str, datetime] | NotGiven = NOT_GIVEN,
        eval_id: str | NotGiven = NOT_GIVEN,
        inputs: object | NotGiven = NOT_GIVEN,
        metadata: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        outputs: object | NotGiven = NOT_GIVEN,
        parent_id: Optional[str] | NotGiven = NOT_GIVEN,
        scores: Iterable[monitoring_log_params.SpanScore] | NotGiven = NOT_GIVEN,
        span_type: Optional[Literal["DETECTOR", "JUDGE", "APP", "MODEL", "FUNCTION", "SCORER"]] | NotGiven = NOT_GIVEN,
        start: Union[str, datetime] | NotGiven = NOT_GIVEN,
        tags: object | NotGiven = NOT_GIVEN,
        user_id: str | NotGiven = NOT_GIVEN,
        root_span: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        return self._post(
            "/monitoring/log",
            body=maybe_transform(
                {
                    "trace_id": trace_id,
                    "id": id,
                    "caller_id": caller_id,
                    "end": end,
                    "eval_id": eval_id,
                    "inputs": inputs,
                    "metadata": metadata,
                    "name": name,
                    "outputs": outputs,
                    "parent_id": parent_id,
                    "scores": scores,
                    "span_type": span_type,
                    "start": start,
                    "tags": tags,
                    "user_id": user_id,
                    "root_span": root_span,
                },
                monitoring_log_params.MonitoringLogParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncMonitoringResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMonitoringResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/haizelabs/haizelabs-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMonitoringResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMonitoringResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/haizelabs/haizelabs-python#with_streaming_response
        """
        return AsyncMonitoringResourceWithStreamingResponse(self)

    @overload
    async def log(
        self,
        *,
        trace_id: str,
        id: str | NotGiven = NOT_GIVEN,
        caller_id: Optional[str] | NotGiven = NOT_GIVEN,
        end: Union[str, datetime] | NotGiven = NOT_GIVEN,
        eval_id: str | NotGiven = NOT_GIVEN,
        inputs: object | NotGiven = NOT_GIVEN,
        metadata: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        outputs: object | NotGiven = NOT_GIVEN,
        parent_id: Optional[str] | NotGiven = NOT_GIVEN,
        scores: Iterable[monitoring_log_params.SpanScore] | NotGiven = NOT_GIVEN,
        span_type: Optional[Literal["DETECTOR", "JUDGE", "APP", "MODEL", "FUNCTION", "SCORER"]] | NotGiven = NOT_GIVEN,
        start: Union[str, datetime] | NotGiven = NOT_GIVEN,
        tags: object | NotGiven = NOT_GIVEN,
        user_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        logs a span

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def log(
        self,
        *,
        root_span: str,
        id: str | NotGiven = NOT_GIVEN,
        end: Union[str, datetime] | NotGiven = NOT_GIVEN,
        eval_id: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        start: Union[str, datetime] | NotGiven = NOT_GIVEN,
        user_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        logs a span

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["trace_id"], ["root_span"])
    async def log(
        self,
        *,
        trace_id: str | NotGiven = NOT_GIVEN,
        id: str | NotGiven = NOT_GIVEN,
        caller_id: Optional[str] | NotGiven = NOT_GIVEN,
        end: Union[str, datetime] | NotGiven = NOT_GIVEN,
        eval_id: str | NotGiven = NOT_GIVEN,
        inputs: object | NotGiven = NOT_GIVEN,
        metadata: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        outputs: object | NotGiven = NOT_GIVEN,
        parent_id: Optional[str] | NotGiven = NOT_GIVEN,
        scores: Iterable[monitoring_log_params.SpanScore] | NotGiven = NOT_GIVEN,
        span_type: Optional[Literal["DETECTOR", "JUDGE", "APP", "MODEL", "FUNCTION", "SCORER"]] | NotGiven = NOT_GIVEN,
        start: Union[str, datetime] | NotGiven = NOT_GIVEN,
        tags: object | NotGiven = NOT_GIVEN,
        user_id: str | NotGiven = NOT_GIVEN,
        root_span: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        return await self._post(
            "/monitoring/log",
            body=await async_maybe_transform(
                {
                    "trace_id": trace_id,
                    "id": id,
                    "caller_id": caller_id,
                    "end": end,
                    "eval_id": eval_id,
                    "inputs": inputs,
                    "metadata": metadata,
                    "name": name,
                    "outputs": outputs,
                    "parent_id": parent_id,
                    "scores": scores,
                    "span_type": span_type,
                    "start": start,
                    "tags": tags,
                    "user_id": user_id,
                    "root_span": root_span,
                },
                monitoring_log_params.MonitoringLogParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class MonitoringResourceWithRawResponse:
    def __init__(self, monitoring: MonitoringResource) -> None:
        self._monitoring = monitoring

        self.log = to_raw_response_wrapper(
            monitoring.log,
        )


class AsyncMonitoringResourceWithRawResponse:
    def __init__(self, monitoring: AsyncMonitoringResource) -> None:
        self._monitoring = monitoring

        self.log = async_to_raw_response_wrapper(
            monitoring.log,
        )


class MonitoringResourceWithStreamingResponse:
    def __init__(self, monitoring: MonitoringResource) -> None:
        self._monitoring = monitoring

        self.log = to_streamed_response_wrapper(
            monitoring.log,
        )


class AsyncMonitoringResourceWithStreamingResponse:
    def __init__(self, monitoring: AsyncMonitoringResource) -> None:
        self._monitoring = monitoring

        self.log = async_to_streamed_response_wrapper(
            monitoring.log,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "TestingUpdateParams",
    "Content",
    "ContentBehavior",
    "ContentInputDetection",
    "ContentInputDetectionDetectorData",
    "ContentInputDetectionDetectorDataTextMatchingDetector",
    "ContentInputDetectionDetectorDataCategoryDetector",
    "ContentInputDetectionDetectorDataNaturalLanguageDetector",
    "ContentInputDetectionDetectorDataComparatorDetector",
    "ContentInputDetectionDetectorDataCustomDetector",
    "ContentInputMessage",
    "ContentInputMessageChatCompletionSystemMessageParam",
    "ContentInputMessageChatCompletionSystemMessageParamContentUnionMember1",
    "ContentInputMessageChatCompletionUserMessageParamInput",
    "ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1",
    "ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartTextParam",
    "ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartImageParam",
    "ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartImageParamImageURL",
    "ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartInputAudioParam",
    "ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio",
    "ContentInputMessageChatCompletionAssistantMessageParamInput",
    "ContentInputMessageChatCompletionAssistantMessageParamInputAudio",
    "ContentInputMessageChatCompletionAssistantMessageParamInputContentUnionMember1",
    "ContentInputMessageChatCompletionAssistantMessageParamInputContentUnionMember1ChatCompletionContentPartTextParam",
    "ContentInputMessageChatCompletionAssistantMessageParamInputContentUnionMember1ChatCompletionContentPartRefusalParam",
    "ContentInputMessageChatCompletionAssistantMessageParamInputFunctionCall",
    "ContentInputMessageChatCompletionAssistantMessageParamInputToolCall",
    "ContentInputMessageChatCompletionAssistantMessageParamInputToolCallFunction",
    "ContentInputMessageChatCompletionToolMessageParam",
    "ContentInputMessageChatCompletionToolMessageParamContentUnionMember1",
    "ContentInputMessageChatCompletionFunctionMessageParam",
    "ContentOutputDetection",
    "ContentOutputDetectionDetectorData",
    "ContentOutputDetectionDetectorDataTextMatchingDetector",
    "ContentOutputDetectionDetectorDataCategoryDetector",
    "ContentOutputDetectionDetectorDataNaturalLanguageDetector",
    "ContentOutputDetectionDetectorDataComparatorDetector",
    "ContentOutputDetectionDetectorDataCustomDetector",
    "ContentOutputMessage",
    "ContentOutputMessageChatCompletionSystemMessageParam",
    "ContentOutputMessageChatCompletionSystemMessageParamContentUnionMember1",
    "ContentOutputMessageChatCompletionUserMessageParamInput",
    "ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1",
    "ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartTextParam",
    "ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartImageParam",
    "ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartImageParamImageURL",
    "ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartInputAudioParam",
    "ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio",
    "ContentOutputMessageChatCompletionAssistantMessageParamInput",
    "ContentOutputMessageChatCompletionAssistantMessageParamInputAudio",
    "ContentOutputMessageChatCompletionAssistantMessageParamInputContentUnionMember1",
    "ContentOutputMessageChatCompletionAssistantMessageParamInputContentUnionMember1ChatCompletionContentPartTextParam",
    "ContentOutputMessageChatCompletionAssistantMessageParamInputContentUnionMember1ChatCompletionContentPartRefusalParam",
    "ContentOutputMessageChatCompletionAssistantMessageParamInputFunctionCall",
    "ContentOutputMessageChatCompletionAssistantMessageParamInputToolCall",
    "ContentOutputMessageChatCompletionAssistantMessageParamInputToolCallFunction",
    "ContentOutputMessageChatCompletionToolMessageParam",
    "ContentOutputMessageChatCompletionToolMessageParamContentUnionMember1",
    "ContentOutputMessageChatCompletionFunctionMessageParam",
]


class TestingUpdateParams(TypedDict, total=False):
    id: str

    contents: Iterable[Content]

    test_id: str

    time: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]


class ContentBehavior(TypedDict, total=False):
    description: Required[str]


class ContentInputDetectionDetectorDataTextMatchingDetector(TypedDict, total=False):
    name: Required[str]

    regex: Required[str]

    user_id: Required[str]

    id: str

    created: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    detector_type: Literal["TEXT_MATCHING"]

    last_updated: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]


class ContentInputDetectionDetectorDataCategoryDetector(TypedDict, total=False):
    category: Required[str]

    name: Required[str]

    user_id: Required[str]

    id: str

    created: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    detector_type: Literal["CATEGORY"]

    last_updated: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]


class ContentInputDetectionDetectorDataNaturalLanguageDetector(TypedDict, total=False):
    name: Required[str]

    natural_language_content: Required[str]

    user_id: Required[str]

    id: str

    created: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    detector_type: Literal["NATURAL_LANGUAGE"]

    last_updated: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]


class ContentInputDetectionDetectorDataComparatorDetector(TypedDict, total=False):
    name: Required[str]

    type: Required[Literal["EXACT_MATCH", "LANGUAGE_MODEL_SIMILARITY"]]

    user_id: Required[str]

    id: str

    created: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    detector_type: Literal["COMPARATOR"]

    last_updated: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]


class ContentInputDetectionDetectorDataCustomDetector(TypedDict, total=False):
    name: Required[str]

    type: Required[Literal["TIERED_DETECTOR"]]

    user_id: Required[str]

    id: str

    created: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    detector_type: Literal["CUSTOM"]

    last_updated: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]


ContentInputDetectionDetectorData: TypeAlias = Union[
    ContentInputDetectionDetectorDataTextMatchingDetector,
    ContentInputDetectionDetectorDataCategoryDetector,
    ContentInputDetectionDetectorDataNaturalLanguageDetector,
    ContentInputDetectionDetectorDataComparatorDetector,
    ContentInputDetectionDetectorDataCustomDetector,
]


class ContentInputDetection(TypedDict, total=False):
    content_id: Required[str]

    detected: Required[bool]

    detector_id: Required[str]

    end_time: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    score: Required[float]

    start_time: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    detector_data: Optional[ContentInputDetectionDetectorData]


class ContentInputMessageChatCompletionSystemMessageParamContentUnionMember1(TypedDict, total=False):
    text: Required[str]

    type: Required[Literal["text"]]


class ContentInputMessageChatCompletionSystemMessageParam(TypedDict, total=False):
    content: Required[Union[str, Iterable[ContentInputMessageChatCompletionSystemMessageParamContentUnionMember1]]]

    role: Required[Literal["system"]]

    name: str


class ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartTextParam(
    TypedDict, total=False
):
    text: Required[str]

    type: Required[Literal["text"]]


class ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartImageParamImageURL(
    TypedDict, total=False
):
    url: Required[str]

    detail: Literal["auto", "low", "high"]


class ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartImageParam(
    TypedDict, total=False
):
    image_url: Required[
        ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartImageParamImageURL
    ]

    type: Required[Literal["image_url"]]


class ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio(
    TypedDict, total=False
):
    data: Required[str]

    format: Required[Literal["wav", "mp3"]]


class ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartInputAudioParam(
    TypedDict, total=False
):
    input_audio: Required[
        ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio
    ]

    type: Required[Literal["input_audio"]]


ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1: TypeAlias = Union[
    ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartTextParam,
    ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartImageParam,
    ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartInputAudioParam,
]


class ContentInputMessageChatCompletionUserMessageParamInput(TypedDict, total=False):
    content: Required[Union[str, Iterable[ContentInputMessageChatCompletionUserMessageParamInputContentUnionMember1]]]

    role: Required[Literal["user"]]

    name: str


class ContentInputMessageChatCompletionAssistantMessageParamInputAudio(TypedDict, total=False):
    id: Required[str]


class ContentInputMessageChatCompletionAssistantMessageParamInputContentUnionMember1ChatCompletionContentPartTextParam(
    TypedDict, total=False
):
    text: Required[str]

    type: Required[Literal["text"]]


class ContentInputMessageChatCompletionAssistantMessageParamInputContentUnionMember1ChatCompletionContentPartRefusalParam(
    TypedDict, total=False
):
    refusal: Required[str]

    type: Required[Literal["refusal"]]


ContentInputMessageChatCompletionAssistantMessageParamInputContentUnionMember1: TypeAlias = Union[
    ContentInputMessageChatCompletionAssistantMessageParamInputContentUnionMember1ChatCompletionContentPartTextParam,
    ContentInputMessageChatCompletionAssistantMessageParamInputContentUnionMember1ChatCompletionContentPartRefusalParam,
]


class ContentInputMessageChatCompletionAssistantMessageParamInputFunctionCall(TypedDict, total=False):
    arguments: Required[str]

    name: Required[str]


class ContentInputMessageChatCompletionAssistantMessageParamInputToolCallFunction(TypedDict, total=False):
    arguments: Required[str]

    name: Required[str]


class ContentInputMessageChatCompletionAssistantMessageParamInputToolCall(TypedDict, total=False):
    id: Required[str]

    function: Required[ContentInputMessageChatCompletionAssistantMessageParamInputToolCallFunction]

    type: Required[Literal["function"]]


class ContentInputMessageChatCompletionAssistantMessageParamInput(TypedDict, total=False):
    role: Required[Literal["assistant"]]

    audio: Optional[ContentInputMessageChatCompletionAssistantMessageParamInputAudio]

    content: Union[str, Iterable[ContentInputMessageChatCompletionAssistantMessageParamInputContentUnionMember1], None]

    function_call: Optional[ContentInputMessageChatCompletionAssistantMessageParamInputFunctionCall]

    name: str

    refusal: Optional[str]

    tool_calls: Iterable[ContentInputMessageChatCompletionAssistantMessageParamInputToolCall]


class ContentInputMessageChatCompletionToolMessageParamContentUnionMember1(TypedDict, total=False):
    text: Required[str]

    type: Required[Literal["text"]]


class ContentInputMessageChatCompletionToolMessageParam(TypedDict, total=False):
    content: Required[Union[str, Iterable[ContentInputMessageChatCompletionToolMessageParamContentUnionMember1]]]

    role: Required[Literal["tool"]]

    tool_call_id: Required[str]


class ContentInputMessageChatCompletionFunctionMessageParam(TypedDict, total=False):
    content: Required[Optional[str]]

    name: Required[str]

    role: Required[Literal["function"]]


ContentInputMessage: TypeAlias = Union[
    ContentInputMessageChatCompletionSystemMessageParam,
    ContentInputMessageChatCompletionUserMessageParamInput,
    ContentInputMessageChatCompletionAssistantMessageParamInput,
    ContentInputMessageChatCompletionToolMessageParam,
    ContentInputMessageChatCompletionFunctionMessageParam,
]


class ContentOutputDetectionDetectorDataTextMatchingDetector(TypedDict, total=False):
    name: Required[str]

    regex: Required[str]

    user_id: Required[str]

    id: str

    created: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    detector_type: Literal["TEXT_MATCHING"]

    last_updated: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]


class ContentOutputDetectionDetectorDataCategoryDetector(TypedDict, total=False):
    category: Required[str]

    name: Required[str]

    user_id: Required[str]

    id: str

    created: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    detector_type: Literal["CATEGORY"]

    last_updated: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]


class ContentOutputDetectionDetectorDataNaturalLanguageDetector(TypedDict, total=False):
    name: Required[str]

    natural_language_content: Required[str]

    user_id: Required[str]

    id: str

    created: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    detector_type: Literal["NATURAL_LANGUAGE"]

    last_updated: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]


class ContentOutputDetectionDetectorDataComparatorDetector(TypedDict, total=False):
    name: Required[str]

    type: Required[Literal["EXACT_MATCH", "LANGUAGE_MODEL_SIMILARITY"]]

    user_id: Required[str]

    id: str

    created: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    detector_type: Literal["COMPARATOR"]

    last_updated: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]


class ContentOutputDetectionDetectorDataCustomDetector(TypedDict, total=False):
    name: Required[str]

    type: Required[Literal["TIERED_DETECTOR"]]

    user_id: Required[str]

    id: str

    created: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    detector_type: Literal["CUSTOM"]

    last_updated: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]


ContentOutputDetectionDetectorData: TypeAlias = Union[
    ContentOutputDetectionDetectorDataTextMatchingDetector,
    ContentOutputDetectionDetectorDataCategoryDetector,
    ContentOutputDetectionDetectorDataNaturalLanguageDetector,
    ContentOutputDetectionDetectorDataComparatorDetector,
    ContentOutputDetectionDetectorDataCustomDetector,
]


class ContentOutputDetection(TypedDict, total=False):
    content_id: Required[str]

    detected: Required[bool]

    detector_id: Required[str]

    end_time: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    score: Required[float]

    start_time: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    detector_data: Optional[ContentOutputDetectionDetectorData]


class ContentOutputMessageChatCompletionSystemMessageParamContentUnionMember1(TypedDict, total=False):
    text: Required[str]

    type: Required[Literal["text"]]


class ContentOutputMessageChatCompletionSystemMessageParam(TypedDict, total=False):
    content: Required[Union[str, Iterable[ContentOutputMessageChatCompletionSystemMessageParamContentUnionMember1]]]

    role: Required[Literal["system"]]

    name: str


class ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartTextParam(
    TypedDict, total=False
):
    text: Required[str]

    type: Required[Literal["text"]]


class ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartImageParamImageURL(
    TypedDict, total=False
):
    url: Required[str]

    detail: Literal["auto", "low", "high"]


class ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartImageParam(
    TypedDict, total=False
):
    image_url: Required[
        ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartImageParamImageURL
    ]

    type: Required[Literal["image_url"]]


class ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio(
    TypedDict, total=False
):
    data: Required[str]

    format: Required[Literal["wav", "mp3"]]


class ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartInputAudioParam(
    TypedDict, total=False
):
    input_audio: Required[
        ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartInputAudioParamInputAudio
    ]

    type: Required[Literal["input_audio"]]


ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1: TypeAlias = Union[
    ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartTextParam,
    ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartImageParam,
    ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1ChatCompletionContentPartInputAudioParam,
]


class ContentOutputMessageChatCompletionUserMessageParamInput(TypedDict, total=False):
    content: Required[Union[str, Iterable[ContentOutputMessageChatCompletionUserMessageParamInputContentUnionMember1]]]

    role: Required[Literal["user"]]

    name: str


class ContentOutputMessageChatCompletionAssistantMessageParamInputAudio(TypedDict, total=False):
    id: Required[str]


class ContentOutputMessageChatCompletionAssistantMessageParamInputContentUnionMember1ChatCompletionContentPartTextParam(
    TypedDict, total=False
):
    text: Required[str]

    type: Required[Literal["text"]]


class ContentOutputMessageChatCompletionAssistantMessageParamInputContentUnionMember1ChatCompletionContentPartRefusalParam(
    TypedDict, total=False
):
    refusal: Required[str]

    type: Required[Literal["refusal"]]


ContentOutputMessageChatCompletionAssistantMessageParamInputContentUnionMember1: TypeAlias = Union[
    ContentOutputMessageChatCompletionAssistantMessageParamInputContentUnionMember1ChatCompletionContentPartTextParam,
    ContentOutputMessageChatCompletionAssistantMessageParamInputContentUnionMember1ChatCompletionContentPartRefusalParam,
]


class ContentOutputMessageChatCompletionAssistantMessageParamInputFunctionCall(TypedDict, total=False):
    arguments: Required[str]

    name: Required[str]


class ContentOutputMessageChatCompletionAssistantMessageParamInputToolCallFunction(TypedDict, total=False):
    arguments: Required[str]

    name: Required[str]


class ContentOutputMessageChatCompletionAssistantMessageParamInputToolCall(TypedDict, total=False):
    id: Required[str]

    function: Required[ContentOutputMessageChatCompletionAssistantMessageParamInputToolCallFunction]

    type: Required[Literal["function"]]


class ContentOutputMessageChatCompletionAssistantMessageParamInput(TypedDict, total=False):
    role: Required[Literal["assistant"]]

    audio: Optional[ContentOutputMessageChatCompletionAssistantMessageParamInputAudio]

    content: Union[str, Iterable[ContentOutputMessageChatCompletionAssistantMessageParamInputContentUnionMember1], None]

    function_call: Optional[ContentOutputMessageChatCompletionAssistantMessageParamInputFunctionCall]

    name: str

    refusal: Optional[str]

    tool_calls: Iterable[ContentOutputMessageChatCompletionAssistantMessageParamInputToolCall]


class ContentOutputMessageChatCompletionToolMessageParamContentUnionMember1(TypedDict, total=False):
    text: Required[str]

    type: Required[Literal["text"]]


class ContentOutputMessageChatCompletionToolMessageParam(TypedDict, total=False):
    content: Required[Union[str, Iterable[ContentOutputMessageChatCompletionToolMessageParamContentUnionMember1]]]

    role: Required[Literal["tool"]]

    tool_call_id: Required[str]


class ContentOutputMessageChatCompletionFunctionMessageParam(TypedDict, total=False):
    content: Required[Optional[str]]

    name: Required[str]

    role: Required[Literal["function"]]


ContentOutputMessage: TypeAlias = Union[
    ContentOutputMessageChatCompletionSystemMessageParam,
    ContentOutputMessageChatCompletionUserMessageParamInput,
    ContentOutputMessageChatCompletionAssistantMessageParamInput,
    ContentOutputMessageChatCompletionToolMessageParam,
    ContentOutputMessageChatCompletionFunctionMessageParam,
]


class Content(TypedDict, total=False):
    id: str

    algorithm: Optional[str]

    behavior: Optional[ContentBehavior]

    content_group_ids: Optional[List[str]]

    content_type: Optional[Literal["BASE", "TEST", "EXPERIMENT"]]

    end: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    generate_end_time: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    generate_start_time: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    input_detections: Optional[Iterable[ContentInputDetection]]

    input_messages: Optional[Iterable[ContentInputMessage]]

    metadata: Optional[object]

    output_detections: Optional[Iterable[ContentOutputDetection]]

    output_messages: Optional[Iterable[ContentOutputMessage]]

    start: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]

    status: Optional[Literal["GENERATING_CONTENT", "AWAITING_RESPONSE", "ANALYZING_RESPONSE", "COMPLETE", "DISCARDED"]]
    """Status for a single content object in a test."""

    test_id: Optional[str]

    time: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    user_id: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import (
    testing_start_params,
    testing_update_params,
    testing_write_score_params,
    testing_create_evaluation_params,
    testing_update_evaluation_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.testing_start_response import TestingStartResponse
from ..types.testing_update_response import TestingUpdateResponse
from ..types.testing_create_evaluation_response import TestingCreateEvaluationResponse

__all__ = ["TestingResource", "AsyncTestingResource"]


class TestingResource(SyncAPIResource):
    __test__ = False

    @cached_property
    def with_raw_response(self) -> TestingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/haizelabs/haizelabs-python#accessing-raw-response-data-eg-headers
        """
        return TestingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TestingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/haizelabs/haizelabs-python#with_streaming_response
        """
        return TestingResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        id: str | NotGiven = NOT_GIVEN,
        contents: Iterable[testing_update_params.Content] | NotGiven = NOT_GIVEN,
        test_id: str | NotGiven = NOT_GIVEN,
        time: Union[str, datetime, None] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestingUpdateResponse:
        """
        Updates an existing test with responses.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/testing/update",
            body=maybe_transform(
                {
                    "id": id,
                    "contents": contents,
                    "test_id": test_id,
                    "time": time,
                },
                testing_update_params.TestingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TestingUpdateResponse,
        )

    def create_evaluation(
        self,
        *,
        eval_type: Literal["EVALUATION", "FUZZING", "RED_TEAMING"],
        name: str,
        user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestingCreateEvaluationResponse:
        """
        Create Evaluation

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/testing/create_evaluation",
            body=maybe_transform(
                {
                    "eval_type": eval_type,
                    "name": name,
                    "user_id": user_id,
                },
                testing_create_evaluation_params.TestingCreateEvaluationParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TestingCreateEvaluationResponse,
        )

    def start(
        self,
        *,
        test_data: testing_start_params.TestData,
        id: str | NotGiven = NOT_GIVEN,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestingStartResponse:
        """
        Starts a new test.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/testing/start",
            body=maybe_transform(
                {
                    "test_data": test_data,
                    "id": id,
                    "user_id": user_id,
                },
                testing_start_params.TestingStartParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TestingStartResponse,
        )

    def update_evaluation(
        self,
        *,
        eval_id: str,
        status: Literal["COMPLETE", "ERROR", "RUNNING", "STOPPED", "PENDING"],
        user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Update Evaluation

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/testing/update_evaluation",
            body=maybe_transform(
                {
                    "eval_id": eval_id,
                    "status": status,
                    "user_id": user_id,
                },
                testing_update_evaluation_params.TestingUpdateEvaluationParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def write_score(
        self,
        *,
        judge_result: testing_write_score_params.JudgeResult,
        span_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Add Score

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/testing/write_score",
            body=maybe_transform(
                {
                    "judge_result": judge_result,
                    "span_id": span_id,
                },
                testing_write_score_params.TestingWriteScoreParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncTestingResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTestingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/haizelabs/haizelabs-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTestingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTestingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/haizelabs/haizelabs-python#with_streaming_response
        """
        return AsyncTestingResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        id: str | NotGiven = NOT_GIVEN,
        contents: Iterable[testing_update_params.Content] | NotGiven = NOT_GIVEN,
        test_id: str | NotGiven = NOT_GIVEN,
        time: Union[str, datetime, None] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestingUpdateResponse:
        """
        Updates an existing test with responses.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/testing/update",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "contents": contents,
                    "test_id": test_id,
                    "time": time,
                },
                testing_update_params.TestingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TestingUpdateResponse,
        )

    async def create_evaluation(
        self,
        *,
        eval_type: Literal["EVALUATION", "FUZZING", "RED_TEAMING"],
        name: str,
        user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestingCreateEvaluationResponse:
        """
        Create Evaluation

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/testing/create_evaluation",
            body=await async_maybe_transform(
                {
                    "eval_type": eval_type,
                    "name": name,
                    "user_id": user_id,
                },
                testing_create_evaluation_params.TestingCreateEvaluationParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TestingCreateEvaluationResponse,
        )

    async def start(
        self,
        *,
        test_data: testing_start_params.TestData,
        id: str | NotGiven = NOT_GIVEN,
        user_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TestingStartResponse:
        """
        Starts a new test.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/testing/start",
            body=await async_maybe_transform(
                {
                    "test_data": test_data,
                    "id": id,
                    "user_id": user_id,
                },
                testing_start_params.TestingStartParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TestingStartResponse,
        )

    async def update_evaluation(
        self,
        *,
        eval_id: str,
        status: Literal["COMPLETE", "ERROR", "RUNNING", "STOPPED", "PENDING"],
        user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Update Evaluation

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/testing/update_evaluation",
            body=await async_maybe_transform(
                {
                    "eval_id": eval_id,
                    "status": status,
                    "user_id": user_id,
                },
                testing_update_evaluation_params.TestingUpdateEvaluationParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def write_score(
        self,
        *,
        judge_result: testing_write_score_params.JudgeResult,
        span_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Add Score

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/testing/write_score",
            body=await async_maybe_transform(
                {
                    "judge_result": judge_result,
                    "span_id": span_id,
                },
                testing_write_score_params.TestingWriteScoreParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class TestingResourceWithRawResponse:
    __test__ = False

    def __init__(self, testing: TestingResource) -> None:
        self._testing = testing

        self.update = to_raw_response_wrapper(
            testing.update,
        )
        self.create_evaluation = to_raw_response_wrapper(
            testing.create_evaluation,
        )
        self.start = to_raw_response_wrapper(
            testing.start,
        )
        self.update_evaluation = to_raw_response_wrapper(
            testing.update_evaluation,
        )
        self.write_score = to_raw_response_wrapper(
            testing.write_score,
        )


class AsyncTestingResourceWithRawResponse:
    def __init__(self, testing: AsyncTestingResource) -> None:
        self._testing = testing

        self.update = async_to_raw_response_wrapper(
            testing.update,
        )
        self.create_evaluation = async_to_raw_response_wrapper(
            testing.create_evaluation,
        )
        self.start = async_to_raw_response_wrapper(
            testing.start,
        )
        self.update_evaluation = async_to_raw_response_wrapper(
            testing.update_evaluation,
        )
        self.write_score = async_to_raw_response_wrapper(
            testing.write_score,
        )


class TestingResourceWithStreamingResponse:
    __test__ = False

    def __init__(self, testing: TestingResource) -> None:
        self._testing = testing

        self.update = to_streamed_response_wrapper(
            testing.update,
        )
        self.create_evaluation = to_streamed_response_wrapper(
            testing.create_evaluation,
        )
        self.start = to_streamed_response_wrapper(
            testing.start,
        )
        self.update_evaluation = to_streamed_response_wrapper(
            testing.update_evaluation,
        )
        self.write_score = to_streamed_response_wrapper(
            testing.write_score,
        )


class AsyncTestingResourceWithStreamingResponse:
    def __init__(self, testing: AsyncTestingResource) -> None:
        self._testing = testing

        self.update = async_to_streamed_response_wrapper(
            testing.update,
        )
        self.create_evaluation = async_to_streamed_response_wrapper(
            testing.create_evaluation,
        )
        self.start = async_to_streamed_response_wrapper(
            testing.start,
        )
        self.update_evaluation = async_to_streamed_response_wrapper(
            testing.update_evaluation,
        )
        self.write_score = async_to_streamed_response_wrapper(
            testing.write_score,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["TestingWriteScoreParams", "JudgeResult"]


class TestingWriteScoreParams(TypedDict, total=False):
    judge_result: Required[JudgeResult]

    span_id: Required[str]


class JudgeResult(TypedDict, total=False):
    judge_id: Required[str]

    score: Required[float]

    judge_name: Optional[str]

    label: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from haizelabs_api import HaizeLabs, AsyncHaizeLabs
from haizelabs_api.types import (
    TestingStartResponse,
    TestingUpdateResponse,
    TestingCreateEvaluationResponse,
)
from haizelabs_api._utils import parse_datetime

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTesting:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_update(self, client: HaizeLabs) -> None:
        testing = client.testing.update()
        assert_matches_type(TestingUpdateResponse, testing, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: HaizeLabs) -> None:
        testing = client.testing.update(
            id="id",
            contents=[
                {
                    "id": "id",
                    "algorithm": "algorithm",
                    "behavior": {"description": "description"},
                    "content_group_ids": ["string"],
                    "content_type": "BASE",
                    "end": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "generate_end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "generate_start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "input_detections": [
                        {
                            "content_id": "content_id",
                            "detected": True,
                            "detector_id": "detector_id",
                            "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "score": 0,
                            "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "detector_data": {
                                "name": "name",
                                "regex": "regex",
                                "user_id": "user_id",
                                "id": "id",
                                "created": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "detector_type": "TEXT_MATCHING",
                                "last_updated": parse_datetime("2019-12-27T18:11:19.117Z"),
                            },
                        }
                    ],
                    "input_messages": [
                        {
                            "content": "string",
                            "role": "system",
                            "name": "name",
                        }
                    ],
                    "metadata": {},
                    "output_detections": [
                        {
                            "content_id": "content_id",
                            "detected": True,
                            "detector_id": "detector_id",
                            "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "score": 0,
                            "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "detector_data": {
                                "name": "name",
                                "regex": "regex",
                                "user_id": "user_id",
                                "id": "id",
                                "created": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "detector_type": "TEXT_MATCHING",
                                "last_updated": parse_datetime("2019-12-27T18:11:19.117Z"),
                            },
                        }
                    ],
                    "output_messages": [
                        {
                            "content": "string",
                            "role": "system",
                            "name": "name",
                        }
                    ],
                    "start": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "status": "GENERATING_CONTENT",
                    "test_id": "test_id",
                    "time": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "user_id": "user_id",
                }
            ],
            test_id="test_id",
            time=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(TestingUpdateResponse, testing, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: HaizeLabs) -> None:
        response = client.testing.with_raw_response.update()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        testing = response.parse()
        assert_matches_type(TestingUpdateResponse, testing, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: HaizeLabs) -> None:
        with client.testing.with_streaming_response.update() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            testing = response.parse()
            assert_matches_type(TestingUpdateResponse, testing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_evaluation(self, client: HaizeLabs) -> None:
        testing = client.testing.create_evaluation(
            eval_type="EVALUATION",
            name="name",
            user_id="user_id",
        )
        assert_matches_type(TestingCreateEvaluationResponse, testing, path=["response"])

    @parametrize
    def test_raw_response_create_evaluation(self, client: HaizeLabs) -> None:
        response = client.testing.with_raw_response.create_evaluation(
            eval_type="EVALUATION",
            name="name",
            user_id="user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        testing = response.parse()
        assert_matches_type(TestingCreateEvaluationResponse, testing, path=["response"])

    @parametrize
    def test_streaming_response_create_evaluation(self, client: HaizeLabs) -> None:
        with client.testing.with_streaming_response.create_evaluation(
            eval_type="EVALUATION",
            name="name",
            user_id="user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            testing = response.parse()
            assert_matches_type(TestingCreateEvaluationResponse, testing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_start(self, client: HaizeLabs) -> None:
        testing = client.testing.start(
            test_data={
                "name": "name",
                "user_id": "user_id",
            },
        )
        assert_matches_type(TestingStartResponse, testing, path=["response"])

    @parametrize
    def test_method_start_with_all_params(self, client: HaizeLabs) -> None:
        testing = client.testing.start(
            test_data={
                "name": "name",
                "user_id": "user_id",
                "id": "id",
                "algorithm_status": [
                    {
                        "name": "name",
                        "status": "COMPLETE",
                    }
                ],
                "behaviors": [{"description": "description"}],
                "content": [
                    {
                        "id": "id",
                        "algorithm": "algorithm",
                        "behavior": {"description": "description"},
                        "content_group_ids": ["string"],
                        "content_type": "BASE",
                        "end": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "generate_end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "generate_start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "input_detections": [
                            {
                                "content_id": "content_id",
                                "detected": True,
                                "detector_id": "detector_id",
                                "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "score": 0,
                                "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "detector_data": {
                                    "name": "name",
                                    "regex": "regex",
                                    "user_id": "user_id",
                                    "id": "id",
                                    "created": parse_datetime("2019-12-27T18:11:19.117Z"),
                                    "detector_type": "TEXT_MATCHING",
                                    "last_updated": parse_datetime("2019-12-27T18:11:19.117Z"),
                                },
                            }
                        ],
                        "input_messages": [
                            {
                                "content": "string",
                                "role": "system",
                                "name": "name",
                            }
                        ],
                        "metadata": {},
                        "output_detections": [
                            {
                                "content_id": "content_id",
                                "detected": True,
                                "detector_id": "detector_id",
                                "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "score": 0,
                                "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "detector_data": {
                                    "name": "name",
                                    "regex": "regex",
                                    "user_id": "user_id",
                                    "id": "id",
                                    "created": parse_datetime("2019-12-27T18:11:19.117Z"),
                                    "detector_type": "TEXT_MATCHING",
                                    "last_updated": parse_datetime("2019-12-27T18:11:19.117Z"),
                                },
                            }
                        ],
                        "output_messages": [
                            {
                                "content": "string",
                                "role": "system",
                                "name": "name",
                            }
                        ],
                        "start": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "status": "GENERATING_CONTENT",
                        "test_id": "test_id",
                        "time": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "user_id": "user_id",
                    }
                ],
                "content_group_ids": ["string"],
                "detector_ids": ["string"],
                "end_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                "prompt_id": "prompt_id",
                "start_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                "status": "COMPLETE",
                "target_model": {
                    "model_id": "model_id",
                    "id": "id",
                    "api_key": "api_key",
                    "base_url": "base_url",
                    "generation_params": {
                        "id": "id",
                        "max_tokens": 1,
                        "system_message": "system_message",
                        "temperature": 0,
                        "user_id": "user_id",
                    },
                    "model_provider": "model_provider",
                    "model_type": "STANDARD",
                    "name": "name",
                    "time": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "user_id": "user_id",
                },
                "test_type": "RED_TEAMING",
            },
            id="id",
            user_id="user_id",
        )
        assert_matches_type(TestingStartResponse, testing, path=["response"])

    @parametrize
    def test_raw_response_start(self, client: HaizeLabs) -> None:
        response = client.testing.with_raw_response.start(
            test_data={
                "name": "name",
                "user_id": "user_id",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        testing = response.parse()
        assert_matches_type(TestingStartResponse, testing, path=["response"])

    @parametrize
    def test_streaming_response_start(self, client: HaizeLabs) -> None:
        with client.testing.with_streaming_response.start(
            test_data={
                "name": "name",
                "user_id": "user_id",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            testing = response.parse()
            assert_matches_type(TestingStartResponse, testing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_update_evaluation(self, client: HaizeLabs) -> None:
        testing = client.testing.update_evaluation(
            eval_id="eval_id",
            status="COMPLETE",
            user_id="user_id",
        )
        assert_matches_type(object, testing, path=["response"])

    @parametrize
    def test_raw_response_update_evaluation(self, client: HaizeLabs) -> None:
        response = client.testing.with_raw_response.update_evaluation(
            eval_id="eval_id",
            status="COMPLETE",
            user_id="user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        testing = response.parse()
        assert_matches_type(object, testing, path=["response"])

    @parametrize
    def test_streaming_response_update_evaluation(self, client: HaizeLabs) -> None:
        with client.testing.with_streaming_response.update_evaluation(
            eval_id="eval_id",
            status="COMPLETE",
            user_id="user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            testing = response.parse()
            assert_matches_type(object, testing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_write_score(self, client: HaizeLabs) -> None:
        testing = client.testing.write_score(
            judge_result={
                "judge_id": "judge_id",
                "score": 0,
            },
            span_id="span_id",
        )
        assert_matches_type(object, testing, path=["response"])

    @parametrize
    def test_method_write_score_with_all_params(self, client: HaizeLabs) -> None:
        testing = client.testing.write_score(
            judge_result={
                "judge_id": "judge_id",
                "score": 0,
                "judge_name": "judge_name",
                "label": "label",
            },
            span_id="span_id",
        )
        assert_matches_type(object, testing, path=["response"])

    @parametrize
    def test_raw_response_write_score(self, client: HaizeLabs) -> None:
        response = client.testing.with_raw_response.write_score(
            judge_result={
                "judge_id": "judge_id",
                "score": 0,
            },
            span_id="span_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        testing = response.parse()
        assert_matches_type(object, testing, path=["response"])

    @parametrize
    def test_streaming_response_write_score(self, client: HaizeLabs) -> None:
        with client.testing.with_streaming_response.write_score(
            judge_result={
                "judge_id": "judge_id",
                "score": 0,
            },
            span_id="span_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            testing = response.parse()
            assert_matches_type(object, testing, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncTesting:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_update(self, async_client: AsyncHaizeLabs) -> None:
        testing = await async_client.testing.update()
        assert_matches_type(TestingUpdateResponse, testing, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncHaizeLabs) -> None:
        testing = await async_client.testing.update(
            id="id",
            contents=[
                {
                    "id": "id",
                    "algorithm": "algorithm",
                    "behavior": {"description": "description"},
                    "content_group_ids": ["string"],
                    "content_type": "BASE",
                    "end": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "generate_end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "generate_start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "input_detections": [
                        {
                            "content_id": "content_id",
                            "detected": True,
                            "detector_id": "detector_id",
                            "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "score": 0,
                            "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "detector_data": {
                                "name": "name",
                                "regex": "regex",
                                "user_id": "user_id",
                                "id": "id",
                                "created": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "detector_type": "TEXT_MATCHING",
                                "last_updated": parse_datetime("2019-12-27T18:11:19.117Z"),
                            },
                        }
                    ],
                    "input_messages": [
                        {
                            "content": "string",
                            "role": "system",
                            "name": "name",
                        }
                    ],
                    "metadata": {},
                    "output_detections": [
                        {
                            "content_id": "content_id",
                            "detected": True,
                            "detector_id": "detector_id",
                            "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "score": 0,
                            "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                            "detector_data": {
                                "name": "name",
                                "regex": "regex",
                                "user_id": "user_id",
                                "id": "id",
                                "created": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "detector_type": "TEXT_MATCHING",
                                "last_updated": parse_datetime("2019-12-27T18:11:19.117Z"),
                            },
                        }
                    ],
                    "output_messages": [
                        {
                            "content": "string",
                            "role": "system",
                            "name": "name",
                        }
                    ],
                    "start": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "status": "GENERATING_CONTENT",
                    "test_id": "test_id",
                    "time": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "user_id": "user_id",
                }
            ],
            test_id="test_id",
            time=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(TestingUpdateResponse, testing, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncHaizeLabs) -> None:
        response = await async_client.testing.with_raw_response.update()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        testing = await response.parse()
        assert_matches_type(TestingUpdateResponse, testing, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncHaizeLabs) -> None:
        async with async_client.testing.with_streaming_response.update() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            testing = await response.parse()
            assert_matches_type(TestingUpdateResponse, testing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_evaluation(self, async_client: AsyncHaizeLabs) -> None:
        testing = await async_client.testing.create_evaluation(
            eval_type="EVALUATION",
            name="name",
            user_id="user_id",
        )
        assert_matches_type(TestingCreateEvaluationResponse, testing, path=["response"])

    @parametrize
    async def test_raw_response_create_evaluation(self, async_client: AsyncHaizeLabs) -> None:
        response = await async_client.testing.with_raw_response.create_evaluation(
            eval_type="EVALUATION",
            name="name",
            user_id="user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        testing = await response.parse()
        assert_matches_type(TestingCreateEvaluationResponse, testing, path=["response"])

    @parametrize
    async def test_streaming_response_create_evaluation(self, async_client: AsyncHaizeLabs) -> None:
        async with async_client.testing.with_streaming_response.create_evaluation(
            eval_type="EVALUATION",
            name="name",
            user_id="user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            testing = await response.parse()
            assert_matches_type(TestingCreateEvaluationResponse, testing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_start(self, async_client: AsyncHaizeLabs) -> None:
        testing = await async_client.testing.start(
            test_data={
                "name": "name",
                "user_id": "user_id",
            },
        )
        assert_matches_type(TestingStartResponse, testing, path=["response"])

    @parametrize
    async def test_method_start_with_all_params(self, async_client: AsyncHaizeLabs) -> None:
        testing = await async_client.testing.start(
            test_data={
                "name": "name",
                "user_id": "user_id",
                "id": "id",
                "algorithm_status": [
                    {
                        "name": "name",
                        "status": "COMPLETE",
                    }
                ],
                "behaviors": [{"description": "description"}],
                "content": [
                    {
                        "id": "id",
                        "algorithm": "algorithm",
                        "behavior": {"description": "description"},
                        "content_group_ids": ["string"],
                        "content_type": "BASE",
                        "end": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "generate_end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "generate_start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "input_detections": [
                            {
                                "content_id": "content_id",
                                "detected": True,
                                "detector_id": "detector_id",
                                "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "score": 0,
                                "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "detector_data": {
                                    "name": "name",
                                    "regex": "regex",
                                    "user_id": "user_id",
                                    "id": "id",
                                    "created": parse_datetime("2019-12-27T18:11:19.117Z"),
                                    "detector_type": "TEXT_MATCHING",
                                    "last_updated": parse_datetime("2019-12-27T18:11:19.117Z"),
                                },
                            }
                        ],
                        "input_messages": [
                            {
                                "content": "string",
                                "role": "system",
                                "name": "name",
                            }
                        ],
                        "metadata": {},
                        "output_detections": [
                            {
                                "content_id": "content_id",
                                "detected": True,
                                "detector_id": "detector_id",
                                "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "score": 0,
                                "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                                "detector_data": {
                                    "name": "name",
                                    "regex": "regex",
                                    "user_id": "user_id",
                                    "id": "id",
                                    "created": parse_datetime("2019-12-27T18:11:19.117Z"),
                                    "detector_type": "TEXT_MATCHING",
                                    "last_updated": parse_datetime("2019-12-27T18:11:19.117Z"),
                                },
                            }
                        ],
                        "output_messages": [
                            {
                                "content": "string",
                                "role": "system",
                                "name": "name",
                            }
                        ],
                        "start": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "status": "GENERATING_CONTENT",
                        "test_id": "test_id",
                        "time": parse_datetime("2019-12-27T18:11:19.117Z"),
                        "user_id": "user_id",
                    }
                ],
                "content_group_ids": ["string"],
                "detector_ids": ["string"],
                "end_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                "prompt_id": "prompt_id",
                "start_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                "status": "COMPLETE",
                "target_model": {
                    "model_id": "model_id",
                    "id": "id",
                    "api_key": "api_key",
                    "base_url": "base_url",
                    "generation_params": {
                        "id": "id",
                        "max_tokens": 1,
                        "system_message": "system_message",
                        "temperature": 0,
                        "user_id": "user_id",
                    },
                    "model_provider": "model_provider",
                    "model_type": "STANDARD",
                    "name": "name",
                    "time": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "user_id": "user_id",
                },
                "test_type": "RED_TEAMING",
            },
            id="id",
            user_id="user_id",
        )
        assert_matches_type(TestingStartResponse, testing, path=["response"])

    @parametrize
    async def test_raw_response_start(self, async_client: AsyncHaizeLabs) -> None:
        response = await async_client.testing.with_raw_response.start(
            test_data={
                "name": "name",
                "user_id": "user_id",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        testing = await response.parse()
        assert_matches_type(TestingStartResponse, testing, path=["response"])

    @parametrize
    async def test_streaming_response_start(self, async_client: AsyncHaizeLabs) -> None:
        async with async_client.testing.with_streaming_response.start(
            test_data={
                "name": "name",
                "user_id": "user_id",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            testing = await response.parse()
            assert_matches_type(TestingStartResponse, testing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_update_evaluation(self, async_client: AsyncHaizeLabs) -> None:
        testing = await async_client.testing.update_evaluation(
            eval_id="eval_id",
            status="COMPLETE",
            user_id="user_id",
        )
        assert_matches_type(object, testing, path=["response"])

    @parametrize
    async def test_raw_response_update_evaluation(self, async_client: AsyncHaizeLabs) -> None:
        response = await async_client.testing.with_raw_response.update_evaluation(
            eval_id="eval_id",
            status="COMPLETE",
            user_id="user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        testing = await response.parse()
        assert_matches_type(object, testing, path=["response"])

    @parametrize
    async def test_streaming_response_update_evaluation(self, async_client: AsyncHaizeLabs) -> None:
        async with async_client.testing.with_streaming_response.update_evaluation(
            eval_id="eval_id",
            status="COMPLETE",
            user_id="user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            testing = await response.parse()
            assert_matches_type(object, testing, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_write_score(self, async_client: AsyncHaizeLabs) -> None:
        testing = await async_client.testing.write_score(
            judge_result={
                "judge_id": "judge_id",
                "score": 0,
            },
            span_id="span_id",
        )
        assert_matches_type(object, testing, path=["response"])

    @parametrize
    async def test_method_write_score_with_all_params(self, async_client: AsyncHaizeLabs) -> None:
        testing = await async_client.testing.write_score(
            judge_result={
                "judge_id": "judge_id",
                "score": 0,
                "judge_name": "judge_name",
                "label": "label",
            },
            span_id="span_id",
        )
        assert_matches_type(object, testing, path=["response"])

    @parametrize
    async def test_raw_response_write_score(self, async_client: AsyncHaizeLabs) -> None:
        response = await async_client.testing.with_raw_response.write_score(
            judge_result={
                "judge_id": "judge_id",
                "score": 0,
            },
            span_id="span_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        testing = await response.parse()
        assert_matches_type(object, testing, path=["response"])

    @parametrize
    async def test_streaming_response_write_score(self, async_client: AsyncHaizeLabs) -> None:
        async with async_client.testing.with_streaming_response.write_score(
            judge_result={
                "judge_id": "judge_id",
                "score": 0,
            },
            span_id="span_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            testing = await response.parse()
            assert_matches_type(object, testing, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from .._models import BaseModel

__all__ = ["JudgeCallResponse"]


class JudgeCallResponse(BaseModel):
    content_id: str

    detected: bool

    end_time: datetime

    judge_id: str

    score: float

    start_time: datetime
